#include "AdDemoView.h"


bool rewardedAdShowRequested = false;
static AdDemoView* viewInstance1 = NULL;
static AdDemoView* viewInstance2 = NULL;

bool AdDemoView::init()
{
    if (!BaseView::init())
        return false;
    
    _coinsEarned = 0;
    
    logo->setPosition(logo->getPosition().x, logo->getPosition().y + 100);
    
    int yPos = logo->getPosition().y - logo->getBoundingBox().size.height / 2;
    
    _txtScreenName = createLabel("<screen name>", 25, Color3B::WHITE, Vec2(visibleSize.width / 2, yPos));
    _txtScreenName->setTextHorizontalAlignment(TextHAlignment::CENTER);
    this->addChild(_txtScreenName);
    
    yPos -= 100;
    
    _txtPointsEarned = BaseView::createLabel("Points Earned: 0", 35, Color3B::BLACK, Vec2(visibleSize.width / 2, yPos));
    _txtPointsEarned->setTextHorizontalAlignment(TextHAlignment::CENTER);
    this->addChild(_txtPointsEarned);
    
    yPos -= 100;
    
    _txtStatus = createLabel("<perform an action>", 25, Color3B::WHITE, Vec2(visibleSize.width / 2, yPos));
    _txtStatus->setTextHorizontalAlignment(TextHAlignment::CENTER);
    this->addChild(_txtStatus);
    
    yPos -= 120;
    
    _btnCacheRewardedAd = createButton("Cache Rewarded Ad", CC_CALLBACK_1(AdDemoView::handleCacheRewardedAd, this), "button_purple.png");
    _btnCacheRewardedAd->setPosition(Vec2(visibleSize.width / 2, yPos));
    this->addChild(_btnCacheRewardedAd);
    
    yPos -= 100;
    
    _btnCacheNonRewardedAd = createButton("Cache Non-Rewared Ad", CC_CALLBACK_1(AdDemoView::handleCacheNonRewardedAd, this));
    _btnCacheNonRewardedAd->setPosition(Vec2(visibleSize.width / 2, yPos));
    this->addChild(_btnCacheNonRewardedAd);
    
    yPos -= 100;
    
    _btnShowRewardedAd = createButton("Show Rewarded Ad", CC_CALLBACK_1(AdDemoView::handleShowRewardedAd, this), "button_purple.png");
    _btnShowRewardedAd->setPosition(Vec2(visibleSize.width / 2, yPos));
    this->addChild(_btnShowRewardedAd);
    
    yPos -= 100;
    
    _btnShowNonRewardedAd = createButton("Show Non-Rewared Ad", CC_CALLBACK_1(AdDemoView::handleShowNonRewardedAd, this));
    _btnShowNonRewardedAd->setPosition(Vec2(visibleSize.width / 2, yPos));
    this->addChild(_btnShowNonRewardedAd);
    
    yPos -= 100;
    
    _btnExportLog = createButton("Export Log", CC_CALLBACK_1(AdDemoView::handleExportLog, this));
    _btnExportLog->setPosition(Vec2(visibleSize.width / 2, yPos));
    _btnExportLog->setScale(0.7f);
    this->addChild(_btnExportLog);
    
    // listen for pokkt events
    PCPokktManager::setAdEventListener(AD_AVAILABILITY_EVENT, PC_AD_EVENT_SELECTOR(AdDemoView::handleAdAvailability), this);
    PCPokktManager::setAdEventListener(AD_CACHING_COMPLETED_EVENT, PC_AD_EVENT_SELECTOR(AdDemoView::handleAdCachingCompleted), this);
    PCPokktManager::setAdEventListener(AD_CACHING_FAILED_EVENT, PC_AD_EVENT_SELECTOR(AdDemoView::handleAdCachingFailed), this);
    PCPokktManager::setAdEventListener(AD_DISPLAYED_EVENT, PC_AD_EVENT_SELECTOR(AdDemoView::handleAdDisplayed), this);
    PCPokktManager::setAdEventListener(AD_SKIPPED_EVENT, PC_AD_EVENT_SELECTOR(AdDemoView::handleAdSkipped), this);
    PCPokktManager::setAdEventListener(AD_COMPLETED_EVENT, PC_AD_EVENT_SELECTOR(AdDemoView::handleAdCompleted), this);
    PCPokktManager::setAdEventListener(AD_CLOSED_EVENT, PC_AD_EVENT_SELECTOR(AdDemoView::handleAdClosed), this);
    PCPokktManager::setAdEventListener(AD_GRATIFIED_EVENT, PC_AD_EVENT_SELECTOR(AdDemoView::handleAdGratified), this);
    
    return true;
}

AdDemoView::~AdDemoView()
{
    PCPokktManager::removeAdEventListener(AD_AVAILABILITY_EVENT);
    PCPokktManager::removeAdEventListener(AD_CACHING_COMPLETED_EVENT);
    PCPokktManager::removeAdEventListener(AD_CACHING_FAILED_EVENT);
    PCPokktManager::removeAdEventListener(AD_DISPLAYED_EVENT);
    PCPokktManager::removeAdEventListener(AD_SKIPPED_EVENT);
    PCPokktManager::removeAdEventListener(AD_COMPLETED_EVENT);
    PCPokktManager::removeAdEventListener(AD_CLOSED_EVENT);
    PCPokktManager::removeAdEventListener(AD_GRATIFIED_EVENT);
}

void AdDemoView::handleBackPress(Ref *pSender)
{
    ChangeViewParams params;
    params.viewToChange = DemoSelectorView;
    changeView(params);
}

void AdDemoView::setAdConfig(PCAdConfig* adConfig)
{
    _adConfig = adConfig;
    _txtScreenName->setString(adConfig->screenName);
}

AdDemoView* AdDemoView::getView(std::string screenName)
{
    if (screenName.compare("DemoScreen1") == 0)
    {
        if (viewInstance1 == NULL)
        {
            viewInstance1 = AdDemoView::create();
            viewInstance1->retain();
        }
        return viewInstance1;
    }
    
    if (screenName.compare("DemoScreen2") == 0)
    {
        if (viewInstance2 == NULL)
        {
            viewInstance2 = AdDemoView::create();
            viewInstance2->retain();
        }
        return viewInstance2;
    }
    
    return NULL;
}


void AdDemoView::handleCacheRewardedAd(Ref *pSender)
{
    _btnCacheRewardedAd->setEnabled(false);
    _btnShowRewardedAd->setEnabled(false);
    _btnCacheRewardedAd->setTitleText("rewarded ad is downloading...");
    _btnShowRewardedAd->setTitleText("rewarded ad is downloading...");
    
    // request pokkt-sdk for a rewarded ad to be cached
    _adConfig->isRewarded = true;
    PCPokktManager::cacheAd(_adConfig);
}

void AdDemoView::handleShowRewardedAd(Ref *pSender)
{
    if (!rewardedAdShowRequested)
    {
        rewardedAdShowRequested = true;
        CCLOG("checking for any available rewarded-ad...");
        _adConfig->isRewarded = true;
        PCPokktManager::checkAdAvailability(_adConfig);
    }
    
    _adConfig->isRewarded = true;
    PCPokktManager::showAd(_adConfig);
}

void AdDemoView::handleCacheNonRewardedAd(Ref *pSender)
{
    _btnCacheNonRewardedAd->setEnabled(false);
    _btnShowNonRewardedAd->setEnabled(false);
    _btnCacheNonRewardedAd->setTitleText("non-rewarded ad is downloading...");
    _btnShowNonRewardedAd->setTitleText("non-rewarded ad is downloading...");
    
    // request pokkt-sdk for a non-rewarded ad to be cached
    _adConfig->isRewarded = false;
    PCPokktManager::cacheAd(_adConfig);
}

void AdDemoView::handleShowNonRewardedAd(Ref *pSender)
{
    // request pokkt-sdk to show a non-rewarded ad
    _adConfig->isRewarded = false;
    PCPokktManager::showAd(_adConfig);
}

void AdDemoView::handleExportLog(Ref* pSender)
{
    PCPokktManager::exportLog();
}

// pokkt handlers
void AdDemoView::handleAdAvailability(PCAdConfig* adConfig, std::map<std::string, std::string> extraValues)
{
    if (adConfig->screenName != _adConfig->screenName)
        return;
    
    // extract values
    bool isAvailable = extraValues[IS_AVAILABLE].compare("true") == 0;
    
    CCLOG("[POKKT-CPP] %s ad for %s is %s!", (adConfig->isRewarded ? "rewarded" : "non-rewarded"), adConfig->screenName.c_str(), (isAvailable ? "available!" : "not available!"));
    
    _txtStatus->setString(isAvailable ? "ad available!" : "no ads available");
    
    if (rewardedAdShowRequested && adConfig->isRewarded)
    {
        PCPokktManager::showAd(adConfig);
        rewardedAdShowRequested = false;
    }
}

void AdDemoView::handleAdCachingCompleted(PCAdConfig* adConfig, std::map<std::string, std::string> extraValues)
{
    if (adConfig->screenName != _adConfig->screenName)
        return;
    
    // extract values
    float vc = 0.0f;
    std::stringstream(extraValues[AD_VC]) >> vc;
    
    CCLOG("%s ad is cached for %s, associated vc is %f", (adConfig->isRewarded ? "rewarded" : "non-rewarded"), adConfig->screenName.c_str(), vc);
    
    _txtStatus->setString("ad cached!");
    
    POKKTLOG("adConfig->isRewarded: %d", adConfig->isRewarded);

    if (adConfig->isRewarded)
    {
        _btnCacheRewardedAd->setTitleText("rewarded ad is cached!");
        _btnShowRewardedAd->setTitleText("Show Rewarded Ad (Cached)");
        _btnShowRewardedAd->setEnabled(true);
    }
    else
    {
        _btnCacheNonRewardedAd->setTitleText("non-rewarded ad is cached!");
        _btnShowNonRewardedAd->setTitleText("Show Non-Rewarded Ad (Cached)");
        _btnShowNonRewardedAd->setEnabled(true);
    }
}

void AdDemoView::handleAdCachingFailed(PCAdConfig* adConfig, std::map<std::string, std::string> extraValues)
{
    if (adConfig->screenName != _adConfig->screenName)
        return;
    
    // extract values
    std::string message = extraValues[FAIL_MESSAGE];
    
    CCLOG("%s ad caching failed for %s! message: %s", (adConfig->isRewarded ? "rewarded" : "non-rewarded"), adConfig->screenName.c_str(), message.c_str());
    
    _txtStatus->setString("ad caching failed!");
    
    if (adConfig->isRewarded)
    {
        _btnCacheRewardedAd->setTitleText("Cache Rewarded Ad");
        _btnShowRewardedAd->setTitleText("Show Rewarded Ad");
        _btnCacheRewardedAd->setEnabled(true);
        _btnShowRewardedAd->setEnabled(true);
    }
    else
    {
        _btnCacheNonRewardedAd->setTitleText("Cache Non-Rewarded Ad");
        _btnShowNonRewardedAd->setTitleText("Show Non-Rewarded Ad");
        _btnCacheNonRewardedAd->setEnabled(true);
        _btnShowNonRewardedAd->setEnabled(true);
    }
}

void AdDemoView::handleAdDisplayed(PCAdConfig* adConfig, std::map<std::string, std::string> extraValues)
{
    if (adConfig->screenName != _adConfig->screenName)
        return;
    
    // ignore extraValue param
    
    CCLOG("%s ad is displayed for %s!", (adConfig->isRewarded ? "rewarded" : "non-rewarded"), adConfig->screenName.c_str());
    
    _txtStatus->setString("ad displayed!");
    
    if (adConfig->isRewarded)
    {
        _btnCacheRewardedAd->setEnabled(false);
        _btnShowRewardedAd->setEnabled(false);
    }
    else
    {
        _btnCacheNonRewardedAd->setEnabled(false);
        _btnShowNonRewardedAd->setEnabled(false);
    }
}

void AdDemoView::handleAdSkipped(PCAdConfig* adConfig, std::map<std::string, std::string> extraValues)
{
    if (adConfig->screenName != _adConfig->screenName)
        return;
    
    // ignore extraValue param
    
    CCLOG("%s ad is skipped for %s!", (adConfig->isRewarded ? "rewarded" : "non-rewarded"), adConfig->screenName.c_str());
    
    _txtStatus->setString("ad skipped!");
}

void AdDemoView::handleAdCompleted(PCAdConfig* adConfig, std::map<std::string, std::string> extraValues)
{
    if (adConfig->screenName != _adConfig->screenName)
        return;
    
    // ignore extraValue param
    
    CCLOG("%s ad is completed for %s!", (adConfig->isRewarded ? "rewarded" : "non-rewarded"), adConfig->screenName.c_str());
    
    _txtStatus->setString("ad completed!");
}

void AdDemoView::handleAdClosed(PCAdConfig* adConfig, std::map<std::string, std::string> extraValues)
{
    if (adConfig->screenName != _adConfig->screenName)
        return;
    
    // extract values
    bool backPressed = extraValues[BACK_PRESSED].compare("true") == 0;
    
    CCLOG("%s ad is closed for %s!", (adConfig->isRewarded ? "rewarded" : "non-rewarded"), adConfig->screenName.c_str());
    
    if (adConfig->isRewarded)
    {
        _btnCacheRewardedAd->setEnabled(true);
        _btnShowRewardedAd->setEnabled(true);
        _btnCacheRewardedAd->setTitleText("Cache Rewarded Ad");
        _btnShowRewardedAd->setTitleText("Show Rewarded Ad");
    }
    else
    {
        _btnCacheNonRewardedAd->setEnabled(true);
        _btnShowNonRewardedAd->setEnabled(true);
        _btnCacheNonRewardedAd->setTitleText("Cache Non-Rewarded Ad");
        _btnShowNonRewardedAd->setTitleText("Show Non-Rewarded Ad");
    }
}

void AdDemoView::handleAdGratified(PCAdConfig* adConfig, std::map<std::string, std::string> extraValues)
{
    if (adConfig->screenName != _adConfig->screenName)
        return;
    
    // extract values
    float reward = 0;
    std::stringstream(extraValues[REWARD]) >> reward;
    
    CCLOG("ad for %s is gratified! reward received: %f!", (adConfig->isRewarded ? "rewarded" : "non-rewarded"), reward);

    _coinsEarned += reward;
    
    char buf[100];
    sprintf(buf, "Points Earned: %f", _coinsEarned);
    _txtPointsEarned->setString(buf);
}
